package cn.ibizlab.humanresource.hrapi.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.humanresource.hrapi.dto.*;
import cn.ibizlab.humanresource.hrapi.mapping.*;
import cn.ibizlab.humanresource.core.base.domain.HROrgAddress;
import cn.ibizlab.humanresource.core.base.service.IHROrgAddressService;
import cn.ibizlab.humanresource.core.base.filter.HROrgAddressSearchContext;
import cn.ibizlab.humanresource.util.annotation.VersionCheck;

@Slf4j
@Api(tags = {"组织地址" })
@RestController("HRApi-hrorgaddress")
@RequestMapping("")
public class HROrgAddressResource {

    @Autowired
    public IHROrgAddressService hrorgaddressService;

    @Autowired
    @Lazy
    public HROrgAddressMapping hrorgaddressMapping;

    @PreAuthorize("hasPermission(this.hrorgaddressMapping.toDomain(#hrorgaddressdto),'iBizHumanResources-HROrgAddress-Create')")
    @ApiOperation(value = "新建组织地址", tags = {"组织地址" },  notes = "新建组织地址")
	@RequestMapping(method = RequestMethod.POST, value = "/hrorgaddresses")
    @Transactional
    public ResponseEntity<HROrgAddressDTO> create(@RequestBody HROrgAddressDTO hrorgaddressdto) {
        HROrgAddress domain = hrorgaddressMapping.toDomain(hrorgaddressdto);
		hrorgaddressService.create(domain);
        HROrgAddressDTO dto = hrorgaddressMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.hrorgaddressMapping.toDomain(#hrorgaddressdtos),'iBizHumanResources-HROrgAddress-Create')")
    @ApiOperation(value = "批量新建组织地址", tags = {"组织地址" },  notes = "批量新建组织地址")
	@RequestMapping(method = RequestMethod.POST, value = "/hrorgaddresses/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<HROrgAddressDTO> hrorgaddressdtos) {
        hrorgaddressService.createBatch(hrorgaddressMapping.toDomain(hrorgaddressdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @VersionCheck(entity = "hrorgaddress" , versionfield = "updatedate")
    @PreAuthorize("hasPermission(this.hrorgaddressService.get(#hrorgaddress_id),'iBizHumanResources-HROrgAddress-Update')")
    @ApiOperation(value = "更新组织地址", tags = {"组织地址" },  notes = "更新组织地址")
	@RequestMapping(method = RequestMethod.PUT, value = "/hrorgaddresses/{hrorgaddress_id}")
    @Transactional
    public ResponseEntity<HROrgAddressDTO> update(@PathVariable("hrorgaddress_id") String hrorgaddress_id, @RequestBody HROrgAddressDTO hrorgaddressdto) {
		HROrgAddress domain  = hrorgaddressMapping.toDomain(hrorgaddressdto);
        domain .setHrorgaddressid(hrorgaddress_id);
		hrorgaddressService.update(domain );
		HROrgAddressDTO dto = hrorgaddressMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.hrorgaddressService.getHrorgaddressByEntities(this.hrorgaddressMapping.toDomain(#hrorgaddressdtos)),'iBizHumanResources-HROrgAddress-Update')")
    @ApiOperation(value = "批量更新组织地址", tags = {"组织地址" },  notes = "批量更新组织地址")
	@RequestMapping(method = RequestMethod.PUT, value = "/hrorgaddresses/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<HROrgAddressDTO> hrorgaddressdtos) {
        hrorgaddressService.updateBatch(hrorgaddressMapping.toDomain(hrorgaddressdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.hrorgaddressService.get(#hrorgaddress_id),'iBizHumanResources-HROrgAddress-Remove')")
    @ApiOperation(value = "删除组织地址", tags = {"组织地址" },  notes = "删除组织地址")
	@RequestMapping(method = RequestMethod.DELETE, value = "/hrorgaddresses/{hrorgaddress_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("hrorgaddress_id") String hrorgaddress_id) {
         return ResponseEntity.status(HttpStatus.OK).body(hrorgaddressService.remove(hrorgaddress_id));
    }

    @PreAuthorize("hasPermission(this.hrorgaddressService.getHrorgaddressByIds(#ids),'iBizHumanResources-HROrgAddress-Remove')")
    @ApiOperation(value = "批量删除组织地址", tags = {"组织地址" },  notes = "批量删除组织地址")
	@RequestMapping(method = RequestMethod.DELETE, value = "/hrorgaddresses/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        hrorgaddressService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PostAuthorize("hasPermission(this.hrorgaddressMapping.toDomain(returnObject.body),'iBizHumanResources-HROrgAddress-Get')")
    @ApiOperation(value = "获取组织地址", tags = {"组织地址" },  notes = "获取组织地址")
	@RequestMapping(method = RequestMethod.GET, value = "/hrorgaddresses/{hrorgaddress_id}")
    public ResponseEntity<HROrgAddressDTO> get(@PathVariable("hrorgaddress_id") String hrorgaddress_id) {
        HROrgAddress domain = hrorgaddressService.get(hrorgaddress_id);
        HROrgAddressDTO dto = hrorgaddressMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "获取组织地址草稿", tags = {"组织地址" },  notes = "获取组织地址草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/hrorgaddresses/getdraft")
    public ResponseEntity<HROrgAddressDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(hrorgaddressMapping.toDto(hrorgaddressService.getDraft(new HROrgAddress())));
    }

    @ApiOperation(value = "检查组织地址", tags = {"组织地址" },  notes = "检查组织地址")
	@RequestMapping(method = RequestMethod.POST, value = "/hrorgaddresses/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody HROrgAddressDTO hrorgaddressdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(hrorgaddressService.checkKey(hrorgaddressMapping.toDomain(hrorgaddressdto)));
    }

    @PreAuthorize("hasPermission(this.hrorgaddressMapping.toDomain(#hrorgaddressdto),'iBizHumanResources-HROrgAddress-Save')")
    @ApiOperation(value = "保存组织地址", tags = {"组织地址" },  notes = "保存组织地址")
	@RequestMapping(method = RequestMethod.POST, value = "/hrorgaddresses/save")
    public ResponseEntity<Boolean> save(@RequestBody HROrgAddressDTO hrorgaddressdto) {
        return ResponseEntity.status(HttpStatus.OK).body(hrorgaddressService.save(hrorgaddressMapping.toDomain(hrorgaddressdto)));
    }

    @PreAuthorize("hasPermission(this.hrorgaddressMapping.toDomain(#hrorgaddressdtos),'iBizHumanResources-HROrgAddress-Save')")
    @ApiOperation(value = "批量保存组织地址", tags = {"组织地址" },  notes = "批量保存组织地址")
	@RequestMapping(method = RequestMethod.POST, value = "/hrorgaddresses/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<HROrgAddressDTO> hrorgaddressdtos) {
        hrorgaddressService.saveBatch(hrorgaddressMapping.toDomain(hrorgaddressdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizHumanResources-HROrgAddress-searchDefault-all') and hasPermission(#context,'iBizHumanResources-HROrgAddress-Get')")
	@ApiOperation(value = "获取DEFAULT", tags = {"组织地址" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/hrorgaddresses/fetchdefault")
	public ResponseEntity<List<HROrgAddressDTO>> fetchDefault(HROrgAddressSearchContext context) {
        Page<HROrgAddress> domains = hrorgaddressService.searchDefault(context) ;
        List<HROrgAddressDTO> list = hrorgaddressMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizHumanResources-HROrgAddress-searchDefault-all') and hasPermission(#context,'iBizHumanResources-HROrgAddress-Get')")
	@ApiOperation(value = "查询DEFAULT", tags = {"组织地址" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/hrorgaddresses/searchdefault")
	public ResponseEntity<Page<HROrgAddressDTO>> searchDefault(@RequestBody HROrgAddressSearchContext context) {
        Page<HROrgAddress> domains = hrorgaddressService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(hrorgaddressMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

